/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

require([
        "aps/load",
        "aps/xhr",
        "aps/ResourceStore",
        "aps/Memory",
        "dojox/mvc/at",
        "dojox/mvc/getStateful",
        "dojox/mvc/getPlainValue",
        "dijit/registry",
        "dojo/promise/all",
        "./js/common.js",
        "./js/messages.js",
        "./js/quotas.js",
        "./js/wizardData.js"
    ],
    function (
        load,
        xhr,
        Store,
        Memory,
        at,
        getStateful,
        getPlainValue,
        registry,
        all,
        common,
        messages,
        quotas,
        WizardData
    ) {
        var firstStepViewId = common.getRelatedViewId('new-select'),
            secondStepViewId = common.getRelatedViewId('new-configure'),
            secondStepData = WizardData.getWizardData(secondStepViewId),
            user = secondStepData.user,
            tenant = aps.context.vars.tenant,
            tenantId = tenant.aps.id,
            backupProfileDefer = xhr.get("/aps/2/resources/" + tenantId + "/profile"),
            quotasStatisticsDefer = xhr.get("/aps/2/resources/" + tenantId + "/quotasStatistics"),
            userStore = new Store({
                apsType: common.TYPES.USERS,
                target: "/aps/2/resources/" + tenantId + "/users"
            }),
            enduserStore = new Store({
                apsType: "http://aps-standard.org/types/core/service-user/1.0",
                target: "/aps/2/resources/"
            }),
            controls = null;

        function resetControls() {
            let getLimitValue = function (limitFieldValue) {
                return limitFieldValue === "" ? common.QUOTA_UNLIMITED_VALUE
                    : limitFieldValue;
            }

            Object.entries(controls).forEach(function ([counterName, control]) {
                control.set("value", getLimitValue(control.value));
            });
        }

        function renderQuotasControls(quotasStatistics, setMaxQuota, userQuota) {
            let quotaContainer = registry.byId("quotaContainer"),
                offeringItemsToDisplay = common.getAvailableCountersForArea('quota_controls_ccpv1_edit_user').filter(function(counter){
                    return counter.name.indexOf('Secondary') < 0;
                });
            controls = quotas.getQuotaControls(null, tenant, offeringItemsToDisplay, quotasStatistics, setMaxQuota, userQuota);

            let controlsList = Object.values(controls);
            controlsList.forEach(function (control) {
                quotaContainer.addChild(control);
            });

            return controlsList.length;
        }

        function hasQuotasToManage(userQuota, quotasStatistics) {
            var availableQuotas = Object.values(quotasStatistics).filter(function (quota) {
                return quota.limit != 0;
            });
            return userQuota != common.USER_QUOTA.OFF && availableQuotas.length;
        }

        function applyQuotaValues(userData, userQuota) {
            let getLimitValue = function (limitFieldValue, isStorage) {
                return limitFieldValue === "" ? common.QUOTA_UNLIMITED_VALUE
                    : limitFieldValue * (isStorage ? common.BYTES_IN_GB : 1);
            }
            if (controls) {
                Object.entries(controls).forEach(function ([counterName, control]) {
                    let offeringItem = common.getUserOfferingItem(userData, counterName, true);
                    if (offeringItem === null) {
                        return;
                    }

                    offeringItem.limit = control.value
                        ? getLimitValue(control.value, common.isStorage(counterName))
                        : userQuota === common.USER_QUOTA.STRICT ? 0 : -1;
                });
            }
        }

        var requests = {
            profiles: backupProfileDefer,
            quotasStatistics: quotasStatisticsDefer,
            listWithAnyExistingUser: xhr.get("/aps/2/resources/" + tenantId + "/users?limit(0,1)")
        };
        if (user.aps.id) {
            requests.user = xhr.get("/aps/2/resources/" + user.aps.id);
        }

        all(requests).then(function (response) {
            var backupProfile = response.profiles.length ? response.profiles[0] : null,
                userQuota = backupProfile ? backupProfile.userQuota : common.USER_QUOTA.STRICT,
                savedUser = response.user,
                quotasStatistics = response.quotasStatistics,
                quotasSectionIsVisible = hasQuotasToManage(userQuota, quotasStatistics),
                userData = secondStepData.userData ? secondStepData.userData : null,
                displayName = user.displayName || savedUser.displayName,
                setMaxQuota = true,
                isFirstUser = !response.listWithAnyExistingUser.length,
                userNotifications = backupProfile && backupProfile.notifications ? backupProfile.notifications : [];

            var userModel = getStateful(userData || {
                offeringItems: [],
                privileged: isFirstUser ? common.USER_PRIVILEGED.YES : common.USER_PRIVILEGED.NO,
                notifications: common.transformNotificationsToObject(userNotifications),
                activateDuringCreation: !savedUser || (savedUser.aps.type.indexOf("http://parallels.com/aps/types/pa/admin-user/") < 0)
            });

            load(["aps/PageContainer", {id: "apsPageContainer"}, [
                ["aps/WizardControl", {
                    id: common.genId('wizardcontrol'),
                    steps: [
                        {viewId: firstStepViewId, text: _("Select user")},
                        {viewId: secondStepViewId, text: _("Configure service"), active: true}
                    ]
                }],
                ["aps/FieldSet", {
                    id: common.genId('fieldset'),
                    title: _("Main")
                }, [
                    ["aps/Output", {
                        id: common.genId('output'),
                        label: _("User name"),
                        value: displayName
                    }],
                    ["aps/CheckBox", {
                        id: "adminPrivileges",
                        label: _("Enable administrator privileges"),
                        checked: at(userModel, "privileged").transform({
                            format: function (value) {
                                return value === common.USER_PRIVILEGED.YES;
                            },
                            parse: function (value) {
                                return (value) ? common.USER_PRIVILEGED.YES : common.USER_PRIVILEGED.NO;
                            }
                        }),
                        hint: _('An administrator can manage services of other users and back up Microsoft 365 seats.')
                    }]
                ]],
                ["aps/FieldSet", {
                    id: common.genId('fieldset'),
                    title: _('Notifications')
                }, [
                    ["aps/FieldSet", {
                        label: _('MANAGEMENT PORTAL'),
                        title: false
                    }],
                    ["aps/CheckBox", {
                        label: _('Quota overuse notifications'),
                        checked: at(userModel.notifications, "quota")
                    }],
                    ["aps/CheckBox", {
                        label: _('Scheduled usage reports'),
                        checked: at(userModel.notifications, "reports")
                    }],
                    ["aps/Output", {
                        class: 'space-before-element'
                    }],
                    ["aps/FieldSet", {
                        label: _('CYBER PROTECTION'),
                        title: false
                    }],
                    ["aps/CheckBox", {
                        label: _('Failure notifications'),
                        checked: at(userModel.notifications, "backup_error")
                    }],
                    ["aps/CheckBox", {
                        label: _('Warning notifications'),
                        checked: at(userModel.notifications, "backup_warning")
                    }],
                    ["aps/CheckBox", {
                        label: _('Success notifications'),
                        checked: at(userModel.notifications, "backup_info")
                    }],
                    ["aps/CheckBox", {
                        label: _('Daily recap about active alerts'),
                        checked: at(userModel.notifications, "backup_daily_report")
                    }]
                ]],
                ["aps/Output", {
                    class: 'space-before-element-block'
                }],
                ["aps/FieldSet", {
                    id: "quotaContainer",
                    visible: quotasSectionIsVisible,
                    description: quotas.getQuotaDescription(userQuota),
                    title: _('Quotas')
                }]
            ]]).then(function () {
                if (quotasSectionIsVisible) {
                    let controlsCount = renderQuotasControls(quotasStatistics, setMaxQuota, userQuota);
                    if (controlsCount === 0) {
                        registry.byId('quotaContainer').set('description', _("No free quotas for assigning."));
                    }
                }

                if (userData) {
                    resetControls();
                }
            });

            aps.app.onSubmit = function () {
                var page = registry.byId("apsPageContainer");
                if (!page.validate()) {
                    aps.apsc.cancelProcessing();
                    return;
                }

                function createAccount(userModel, user) {
                    userModel.user = [user];
                    if (Object.keys(userModel.notifications).length) {
                        userModel.notifications = common.transformNotificationsToArray(userModel.notifications);
                    } else {
                        delete userModel.notifications;
                    }
                    userStore.put(userModel).then(function () {
                        WizardData.resetWizardData(firstStepViewId);
                        WizardData.resetWizardData(secondStepViewId);
                        aps.apsc.gotoView(common.getRelatedViewId('list'), tenant.aps.id);
                    }, messages.displayError);
                }

                applyQuotaValues(userModel, userQuota);
                userModel.aps = {type: common.TYPES.USERS};
                userModel.tenant = [{aps: {id: tenantId}}];

                xhr.get("/aps/2/resources/" + tenant.aps.id + "/loginExists", {
                    query: {
                        login: user.login ? user.login : savedUser.login
                    }
                }).then(function (existing) {
                    if (existing) {
                        messages.removeAll();
                        messages.displayWarning(_("The account with email address already exists in the Cyber Protection service. Delete the old account or specify a different service user."));
                        aps.apsc.cancelProcessing();
                        return;
                    }
                    if (user.aps.id) {
                        createAccount(userModel, user);
                    } else {
                        enduserStore.put(user).then(function (enduser) {
                            user = {aps: {id: enduser.aps.id}};
                            createAccount(userModel, user);
                        }, messages.displayError);
                    }
                }, messages.displayError);
            };

            aps.app.onPrev = function () {
                applyQuotaValues(userModel, userQuota);
                secondStepData.userData = getPlainValue(userModel);
                WizardData.setWizardData(secondStepData);
                aps.apsc.gotoView(firstStepViewId);
            };
        }, messages.displayError);

        aps.app.onCancel = function () {
            WizardData.resetWizardData(firstStepViewId);
            WizardData.resetWizardData(secondStepViewId);
            aps.apsc.gotoView(common.getRelatedViewId('list'), tenant.aps.id);
        };
    });
