/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "dojo/_base/declare",
    "dojo/promise/all",
    "dojo/Deferred",
    "aps/xhr",
    "aps/confirm",
    "aps/ResourceStore",
    "./../../common"
], function (
    declare,
    all,
    Deferred,
    xhr,
    confirm,
    Store,
    common
) {
    function getBackupAccount(tenant, user) {
        var deferred = new Deferred();
        var store = new Store({target: "/aps/2/resources/" + tenant.aps.id + "/users"});
        store.query({"user.aps.id": user.aps.id}).then(function (users) {
            users && users.length ? deferred.resolve(users[0]) : deferred.resolve(null);
        }, deferred.reject);
        return deferred;
    }

    return declare(null, {
        user: null,
        tenant: null,
        plugin: null,

        constructor: function (user, tenant, plugin) {
            this.user = user;
            this.tenant = tenant;
            this.plugin = plugin;
        },

        _gotoView: function() {
            var self = this;
            if (self.plugin) {
                return self.plugin.nav.gotoView.apply(self.plugin.nav, arguments);
            }
            return aps.apsc.gotoView.apply(aps.apsc, arguments);
        },

        _showPopup: function() {
            var self = this;
            if (self.plugin) {
                return self.plugin.nav.showPopup.apply(self.plugin.nav, arguments);
            }
            return aps.apsc.showPopup.apply(aps.apsc, arguments);
        },

        assign: function () {
            var self = this,
                deferred = new Deferred();

            self._gotoView("http://www.acronis.com/aps/cloud-backup-v1#nextcp-ccp-setquotas", null, {
                userId: self.user.aps.id,
                isNewUser: true
            });
            deferred.resolve(true);
            return deferred;
        },

        configure: function () {
            var self = this,
                deferred = new Deferred();

            self._gotoView("http://www.acronis.com/aps/cloud-backup-v1#nextcp-ccp-setquotas", null, {
                userId: self.user.aps.id,
                isNewUser: false
            });
            deferred.resolve(true);
            return deferred;
        },

        unassign: function () {

            var self = this,
                deferred = new Deferred();
            getBackupAccount(this.tenant, this.user).then(function (backupAccount) {
                if (!backupAccount) {
                    deferred.reject();
                    return;
                }

                confirm({
                    title: _("Do you want to unassign Cyber Protection service from {0}?", {}, {}, self.plugin).replace("{0}", backupAccount.displayName),
                    description: _("This operation will permanently remove the service user and all his backups", {}, {}, self.plugin),
                    submitType: 'danger',
                    submitLabel: _("Unassign", {}, {}, self.plugin),
                    cancelLabel: _("Cancel", {}, {}, self.plugin)
                }).then(function (doDelete) {
                    if (!doDelete) {
                        deferred.resolve(false);
                        return;
                    }

                    var store = new Store({target: "/aps/2/resources"});
                    store.remove(backupAccount.aps.id).then(function () {
                        deferred.resolve(true);
                    }, deferred.reject);
                }, deferred.reject);
            }, deferred.reject);
            return deferred;
        },

        activate: function () {
            var self = this,
                deferred = new Deferred();
            getBackupAccount(this.tenant, this.user).then(function (backupAccount) {
                if (!backupAccount) {
                    deferred.reject();
                    return;
                }
                self._showPopup({
                    viewId: "http://www.acronis.com/aps/cloud-backup-v1#nextcp-ccp-activate",
                    params: {enduser: backupAccount},
                    modal: true
                }).then(function (callback) {
                    deferred.resolve(callback.btnType === "submit");
                }, deferred.reject);
            }, deferred.reject);
            return deferred;
        },

        changePassword: function () {
            var self = this,
                deferred = new Deferred();
            getBackupAccount(this.tenant, this.user).then(function (backupAccount) {
                if (!backupAccount) {
                    deferred.reject();
                    return;
                }
                self._showPopup({
                    viewId: "http://www.acronis.com/aps/cloud-backup-v1#nextcp-ccp-password",
                    params: {enduser: backupAccount},
                    modal: true
                }).then(function (callback) {
                    deferred.resolve(callback.btnType === "submit");
                }, deferred.reject);
            }, deferred.reject);
            return deferred;
        },

        loginToConsole: function () {
            var deferred = new Deferred();
            getBackupAccount(this.tenant, this.user).then(function (backupAccount) {
                if (!backupAccount) {
                    deferred.reject();
                    return;
                }
                xhr.get("/aps/2/resources/" + backupAccount.aps.id + "/loginByToken").then(function (uri) {
                    deferred.resolve(uri);
                }, deferred.reject);
            }, deferred.reject);
            return deferred;
        },
        enable: function () {
            var deferred = new Deferred();
            getBackupAccount(this.tenant, this.user).then(function (backupAccount) {
                if (!backupAccount) {
                    deferred.reject();
                    return;
                }
                xhr("/aps/2/resources/" + backupAccount.aps.id + "/", {
                    method: "PUT",
                    headers: { "Content-Type": "application/json" },
                    data: JSON.stringify({"servicestatus": common.STATUS.ENABLED})
                }).then(deferred.resolve, deferred.reject);
            }, deferred.reject);
            return deferred;
        },
        disable: function () {
            var deferred = new Deferred();
            getBackupAccount(this.tenant, this.user).then(function (backupAccount) {
                if (!backupAccount) {
                    deferred.reject();
                    return;
                }
                xhr("/aps/2/resources/" + backupAccount.aps.id + "/", {
                    method: "PUT",
                    headers: { "Content-Type": "application/json" },
                    data: JSON.stringify({"servicestatus": common.STATUS.DISABLED})
                }).then(deferred.resolve, deferred.reject);
            }, deferred.reject);
            return deferred;
        }
    });
});
