/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "dojo/promise/all",
    "aps/Button",
    "aps/Container",
    "aps/DropDownButton",
    "aps/Grid",
    "aps/Memory",
    "aps/Status",
    "aps/Toolbar",
    "aps/xhr",
    "./../../common",
    "./../../messages",
    "./../../wizardData"
], function (
    all,
    Button,
    Container,
    DropDownButton,
    Grid,
    Memory,
    Status,
    Toolbar,
    xhr,
    common,
    messages,
    wizardData
) {

    var statusInfo = {};
    statusInfo[common.STATUS.INACTIVE] = {"label": _("NOT ACTIVATED"), "type": "warning", isLoad: false};
    statusInfo[common.STATUS.ENABLED] = {"label": _("ACTIVATED"), "type": "success", isLoad: false};
    statusInfo[common.STATUS.DISABLED] = {"label": _("DISABLED"), "type": "default", isLoad: true};

    function getAdminGrid(tenant) {
        function createStatus(statusName) {
            return new Status({
                useIcon: false,
                gridSize: 'md-12',
                status: statusName,
                statusInfo: statusInfo
            });
        }
        function getActionsMenu(grid, row) {

            var resendActivateMenuItem = {
                    iconClass: "fa-user",
                    label: _("Resend activation email"),
                    onClick: function () {
                        messages.removeAll();
                        xhr("/aps/2/resources/" + tenant.aps.id + "/administrators/" + row.id + "/resend", {
                            method: "PUT",
                            query: {datacenterId: tenant.datacenterId}
                        }).then(
                            function () {
                                messages.displayMessage(_("The activation message has been successfully sent to {0}.").replace("{0}", row.email), 'info');
                            },
                            function () {
                                messages.displayMessage(_("Failed to send the activation message to {0}.").replace("{0}", row.login), 'error');
                            }
                        );
                    }
                },
                disableMenuItem = {
                    iconClass: "fa-pencil",
                    label: _("Disable"),
                    onClick: function () {
                        messages.removeAll();
                        xhr("/aps/2/resources/" + tenant.aps.id + "/administrators/" + row.id + "/disable", {
                            method: "PUT",
                            query: {datacenterId: row.datacenterId}
                        }).then(
                            function () {
                                messages.displayMessage(_("Administrator {0} has been successfully disabled.").replace("{0}", row.login), 'info');
                                xhr.get("/aps/2/resources/" + tenant.aps.id + "/administrators").then(function (administrators) {
                                    grid.store.setData(administrators);
                                    grid.refresh();
                                }, messages.displayError);
                            },
                            function () {
                                messages.displayMessage(_("Failed to disable adminisrtator {0}.").replace("{0}", row.login), 'error');
                            }
                        );
                    }
                },
                enableMenuItem = {
                    iconClass: "fa-pencil",
                    label: _("Enable"),
                    onClick: function () {
                        messages.removeAll();
                        xhr("/aps/2/resources/" + tenant.aps.id + "/administrators/" + row.id + "/enable", {
                            method: "PUT",
                            query: {datacenterId: tenant.datacenterId}
                        }).then(
                            function () {
                                messages.displayMessage(_("Administrator {0} has been successfully enabled.").replace("{0}", row.login), 'info');
                                xhr.get("/aps/2/resources/" + tenant.aps.id + "/administrators").then(function (administrators) {
                                    grid.store.setData(administrators);
                                    grid.refresh();
                                }, messages.displayError);
                            },
                            function () {
                                messages.displayMessage(_("Failed to enable administrator {0}.").replace("{0}", row.login), 'error');
                            }
                        );
                    }
                },
                menuItems = [];

            switch (row.status.toString()) {
                case common.STATUS.INACTIVE:
                    menuItems = [resendActivateMenuItem];
                    break;

                case common.STATUS.DISABLED:
                    menuItems = [enableMenuItem];
                    break;

                case common.STATUS.ENABLED:
                    menuItems = [disableMenuItem];
                    break;
            }

            return new DropDownButton({
                iconClass: "fa-cog",
                label: _("Actions"),
                items: menuItems
            });
        }

        return xhr.get("/aps/2/resources/" + tenant.aps.id + "/administrators").then(function (response) {
            var adminStore = new Memory({data: response, idProperty: "id"}),
                grid = null,
                layout = [
                    {
                        name: _('Login'),
                        field: "login",
                        type: "resourceName",
                        filter: true
                    },
                    {
                        name: _("Email"),
                        field: "email",
                        sortable: false
                    },
                    {
                        name: _("Status"),
                        field: "status",
                        sortable: false,
                        renderCell: function (row, data) {
                            return createStatus(data);
                        }
                    },
                    {
                        name: '',
                        field: "backupUser",
                        sortable: false,
                        renderCell: function (row) {
                            return getActionsMenu(grid, row);
                        }
                    }
                ],
                toolbar = new Toolbar(),
                addButton = new Button({
                    autoBusy: false,
                    label: _("Add New Administrator"),
                    iconClass: "fa-plus",
                    isDefault: true,
                    onClick: function () {
                        aps.apsc.gotoView('http://www.acronis.com/aps/cloud-backup-v1#nextcp-ccp-administrator-new', null, {});
                    }
                }),
                deleteButton = new Button({
                    autoBusy: false,
                    label: _("Delete Administrators"),
                    iconClass: "fa-times",
                    onClick: function () {
                        var self = this;
                        if (!confirm(_("Are you sure that you want to remove the administrator account?"))) {
                            self.cancel();
                            return;
                        }

                        var selectionArray = grid.get("selectionArray"),
                            defers = [],
                            deletedIds = [];

                        messages.removeAll();

                        selectionArray.forEach(function (id) {
                            var admin = grid.store.get(id),
                                deffer = xhr("/aps/2/resources/" + tenant.aps.id + "/administrators/" + admin.id, {
                                    method: "DELETE",
                                    query: {
                                        datacenterId: ''
                                    }
                                }).then(function () {
                                    deletedIds.push(id);
                                }, messages.displayErrorAndGo);
                            defers.push(deffer);
                        });

                        all(defers).then(function () {
                            deletedIds.forEach(function (id) {
                                grid.store.remove(id);
                                selectionArray.splice(selectionArray.indexOf(id), 1);
                            });
                            grid.refresh();
                            self.cancel();
                        });
                    }
                });
            grid = new Grid({
                selectionMode: "multiple",
                columns: layout,
                store: adminStore,
                showPaging: true,
                rowsPerPage: 20,
                pageSizeOptions: [20],
                onRowClick: function (row) {
                    aps.apsc.gotoView('http://www.acronis.com/aps/cloud-backup-v1#nextcp-ccp-administrator-edit', null, {"id": row.id});
                }
            });

            toolbar.addChild(addButton);
            toolbar.addChild(deleteButton);
            grid.addChild(toolbar);

            var data = wizardData.getWizardData('nextcp-ccp-partner');
            if (data && data.message) {
                messages.displayMessage(data.message.text, data.message.type);
                wizardData.resetWizardData('nextcp-ccp-partner');
            }

            return grid;
        });
    }

    return getAdminGrid;
});

