/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
	"dojo/_base/declare",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "dojox/mvc/getPlainValue",
	"aps/_View",
	"aps/load",
    "aps/xhr",
    "./../../messages",
    "./../../common"
], function (declare, registry, at, getStateful, getPlainValue, _View, load, xhr, messages, common) {

	return declare(_View, {
		tenant: getStateful({
			aps:{type: common.TYPES.TENANT}
		}),
		init: function () {
            var self = this,
                servicePlanId;

            if (typeof aps.context.requestParams.id === 'undefined') {
                messages.displayError(_("The service plan ID is required."));
                return;
            }

            servicePlanId = aps.context.requestParams.id;

            xhr.get("/aps/2/resources/" + aps.context.vars.product_catalog.aps.id + "/pc/plans?planId=" + servicePlanId).then(function(response){
                if (response.length > 0) {
                    var servicePan = response[0];
                    load([
                        ["aps/Container", { "class": 'lead'}, [
                            ["aps/Output", {
                                value: _('Acronis Cyber Cloud')
                            }]
                        ]],
                        ["aps/Panel", {
                            title: servicePan.name,
                            gridSize: 'md-12'
                        }, [
                            ["aps/Output", {
                                escapeHTML: false,
                                content: servicePan.longDescription
                            }]
                        ]]
                    ], self.domNode);
                }
                else {
                    messages.displayError(_("The service plan with ID \"{0}\" is not found.").replace("{0}", servicePlanId));
                }
            }).otherwise(messages.displayError);

		},

        onHide: function() {
            messages.removeAll();
        },

		onContext: function(){
			aps.apsc.hideLoading();
		},

		onNext: function(){
			var self = this;
            messages.removeAll();
			aps.apsc.next(getPlainValue(self.tenant));
		},

		onPrev: function(){
			aps.apsc.prev();
		}
	});
});