/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

define([
    'dojo/_base/declare',
    'dojo/promise/all',
    'aps/_View',
    'aps/xhr',
    './../components/BackupUserActions',
    './../components/getDownloadTile',
    './../components/getUserResourcesTile',
    './../../common',
    './../../messages'
], function (
    declare,
    all,
    _View,
    xhr,
    BackupUserActions,
    getDownloadTile,
    getUserResourcesTile,
    common,
    messages
) {
    return declare(_View, {
        _backupTilesViewId: null,
        init: function () {
            var self = this;

            self._backupTilesViewId = self.genId('backup_tiles');
            return [
                ['aps/FieldSet', {
                    showLabels: false,
                    id: self.genId('backup_welcome')
                }, [
                    ['aps/Output', {
                        value: _('Welcome to Acronis Cyber Cloud, a unique integration of backup with full-stack next-generation anti-malware protection and comprehensive endpoint management tools. This synergy eliminates complexity and improves security so you can have better protection for your data.'),
                        id: self.genId('backup_welcome_ouput')
                    }]
                ]],
                ['aps/Tiles', {
                    id: self._backupTilesViewId
                }]
            ];
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function () {
            var self = this,
                backupAccount = aps.context.vars.enduser,
                user = aps.context.user,
                backupTiles = self.byId(self._backupTilesViewId),
                statisticsURI = '/aps/2/resources/' + backupAccount.aps.id + '/updateStatistics',
                tenantURI = '/aps/2/resources/' + backupAccount.aps.id + '/getTenant';

            /**
             * Statistics Widget
             *
             * @param backupAccountStatistic
             * @param tenant
             */
            var drawStatistics = function(backupAccountStatistic, tenant) {
                let backupAccountActions = new BackupUserActions(backupAccount),
                    countersList = common.getAvailableCountersForArea('user_quotas');

                backupTiles.removeAll();
                backupTiles.addChild(
                    getUserResourcesTile(
                        backupAccount,
                        backupAccountStatistic,
                        backupAccountActions,
                        tenant,
                        countersList,
                        user
                    ));
            };

            all({
                backupAccountStatistic: xhr.get(statisticsURI),
                tenant: xhr.get(tenantURI)
            }).then(function (response) {
                drawStatistics(response.backupAccountStatistic, response.tenant);

                if (backupAccount && backupAccount.userstatus === common.USER_STATUS.ACTIVE) {
                    backupTiles.addChild(getDownloadTile('nextcp-mycp-download'));
                }

                aps.apsc.hideLoading();
            }, messages.displayError);
        }
    });
});

