/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "aps/Memory",
    "aps/_View",
    "aps/xhr",
    "aps/FieldSet",
    "dojo/_base/declare",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful",
    "./../../../common",
    "./../../../messages"
], function (
    Memory,
    _View,
    xhr,
    FieldSet,
    declare,
    registry,
    at,
    getPlainValue,
    getStateful,
    common,
    messages
) {

    return declare(_View, {
        _adminModel: null,
        _tenant: null,

        init: function () {
            var self = this;
            var adminModel = getStateful({
                    login: "",
                    email: "",
                    firstName: "",
                    lastName: ""
                }),
                enableOptions = [
                    {value: "1", label: _("Enabled")},
                    {value: "0", label: _("Disabled")}
                ];
            self._adminModel = adminModel;

            return [
                ["aps/Panel", [
                    ["aps/FieldSet", {title: _("General information")}, [
                        ["aps/FieldSet",[
                            ["aps/TextBox", {
                                id: self.genId('login'),
                                label: _("Login"),
                                required: true,
                                trim: true,
                                value: at(adminModel, "login")
                            }],
                            ["aps/TextBox", {
                                id: self.genId('email'),
                                label: _("Email"),
                                required: true,
                                trim: true,
                                pattern: '^[^\\s@]+@[^\\s@]+\\.[^\\s@.]+$',
                                invalidMessage: _('Please enter a valid email.'),
                                value: at(adminModel, "email")
                            }]
                        ]]
                    ]],
                    ["aps/FieldSet", {title: _("Optional")}, [
                        ["aps/FieldSet",[
                            ["aps/TextBox", {
                                id: self.genId("first_name"),
                                label: _("First name"),
                                required: false,
                                optionalText: "",
                                trim: true,
                                value: at(adminModel, "firstName")
                            }],
                            ["aps/TextBox", {
                                id: self.genId("last_name"),
                                label: _("Last name"),
                                required: false,
                                optionalText: "",
                                trim: true,
                                value: at(adminModel, "lastName")
                            }]
                        ]]
                    ]]
                ]]
            ];
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function (context) {
            var self = this;
            self._tenant = context.vars.tenant;
            var profile = self._tenant.profile;

            aps.apsc.hideLoading();
        },
        onSubmit: function () {
            var self = this,
                tenant = self._tenant,
                adminModel = self._adminModel;

            if (!self.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            xhr.get("/aps/2/resources/" + tenant.aps.id + "/loginExists", {
                query: {
                    login: adminModel.login,
                    datacenterId: ''
                }
            }).then(function (existing) {
                if (existing) {
                    messages.displayWarning(_("This login already exists."));
                    aps.apsc.cancelProcessing();
                    return;
                }
                xhr("/aps/2/resources/" + tenant.aps.id + "/administrators/", {
                    method: "POST",
                    query: {
                        datacenterId: ''
                    },
                    data: JSON.stringify(getPlainValue(adminModel)),
                    headers: {"Content-Type": "application/json"}
                }).then(function () {
                    aps.apsc.gotoView("nextcp-ccp-partner");
                }, messages.displayError);
            }, messages.displayError);
        }
    });
});
