/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

require([
    "app/common",
    "app/messages",
    "aps/load",
    "aps/ResourceStore",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful"
], function (
    common,
    messages,
    load,
    Store,
    registry,
    at,
    getStateful,
    getPlainValue
) {
    var datacentersStore = new Store({
            target: "/aps/2/resources/" + aps.context.vars.globals.aps.id + "/datacenters"
        }),
        datacenterModel = getStateful({
            aps: {type: common.TYPES.DATACENTER},
            name: "",
            url: "https://cloud.acronis.com",
            login: "",
            password: ""
        });

    var widgets = ["aps/PageContainer", {id: "apsPageContainer"}, [
        ["aps/FieldSet", {id: common.genId('fieldset'), title: _("General")}, [
            ["aps/TextBox", {
                id: "name",
                label: _("Name"),
                value: at(datacenterModel, "name"),
                required: true,
                trim: true
            }],
            ["aps/TextBox", {
                id: "url",
                label: _("URL"),
                value: at(datacenterModel, "url"),
                required: true,
                trim: true
            }],
            ["aps/TextBox", {
                id: "login",
                label: _("Login"),
                value: at(datacenterModel, "login"),
                required: true,
                trim: true
            }],
            ["aps/Password", {
                id: "password",
                label: _("Password"),
                value: at(datacenterModel, "password"),
                required: true,
                showShowPasswordButton: false,
                showStrengthIndicator: false,
                showResetButton: false,
                confirmation: false,
                autoSize: true,
                size: 8
            }]
        ]]
    ]];
    load(widgets).then(function () {
        aps.app.onSubmit = function () {
            var page = registry.byId("apsPageContainer");
            if (!page.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }
            if (/^(http|https):\/\//.exec(datacenterModel.url) == null) {
                datacenterModel.set('url', 'https://' + datacenterModel.url);
            }
            datacentersStore.put(getPlainValue(datacenterModel)).then(function () {
                aps.apsc.gotoView(common.getRelatedViewId('list'));
            }, messages.displayError);
        };
    }, messages.displayError);

    aps.app.onCancel = function () {
        aps.apsc.gotoView(common.getRelatedViewId('list'));
    };
});
