<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Console\Command;

use Acronis\App\Services\Apsc\ApscInstanceFactoryTrait;
use APS\ControllerProxy;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

abstract class ApsInstanceCommand extends Loggable
{
    use ApscInstanceFactoryTrait;

    const ARGUMENT_APS_INSTANCE_ID = 'apsInstanceId';
    const ARGUMENT_APS_INSTANCE_ID_DESCRIPTION = 'APS Instance ID (UUID).';

    /** @var ControllerProxy */
    private $apsc;

    /** @var string */
    private $apsInstanceId;

    protected function configure()
    {
        parent::configure();

        $this->addArgument(
            static::ARGUMENT_APS_INSTANCE_ID,
            InputArgument::REQUIRED,
            static::ARGUMENT_APS_INSTANCE_ID_DESCRIPTION
        );
    }

    /**
     * @return string
     */
    protected function getApsInstanceId()
    {
        return $this->apsInstanceId;
    }

    /**
     * @return ControllerProxy
     */
    protected function getApsc()
    {
        return $this->apsc;
    }

    /** Exception won't be thrown in CLI mode */
    /** @noinspection PhpDocMissingThrowsInspection */
    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int|null|void
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $this->apsInstanceId = $input->getArgument(static::ARGUMENT_APS_INSTANCE_ID);
        /** @noinspection PhpUnhandledExceptionInspection */
        $this->apsc = $this->createApscInstance($this->apsInstanceId);
    }
}