<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */
class Model_MinimalCommitmentDiscountManager implements Model_MinimalCommitmentDiscountManagerInterface
{
    /**
     * @var Service_PbaApi_ApiInterface
     */
    private $_pbaApi;

    /**
     * @var Service_OpenApi_ApiInterface
     */
    private $_openApi;

    const REQUIRED_DISCOUNT_RATE_FOR_FIRST_LEVEL_RESELLER = 100;

    /**
     * Model_MinimalCommitmentDiscountManager constructor.
     * @param Service_OpenApi_ApiInterface $openApi
     * @param Service_PbaApi_ApiInterface $pbaApi
     */
    public function __construct(
        Service_OpenApi_ApiInterface $openApi,
        Service_PbaApi_ApiInterface $pbaApi
    ) {
        $this->_openApi = $openApi;
        $this->_pbaApi = $pbaApi;
    }

    /**
     * @param int $discountId
     * @return null|Service_PbaApi_Api_Response_Struct_Discount
     * @throws Service_PbaApi_Api_Exception
     */
    public function fetchDiscountById($discountId)
    {
        try {
            $discount = $this->_pbaApi->getDiscountById((int)$discountId);
        } catch (Service_PbaApi_Api_Exception $e) {
            if ($e->isXmlRpcException()) {
                return null;
            }
            throw $e;
        }
        return $discount;
    }

    /**
     * @param int $accountId
     * @return Service_OpenApi_Response_AccountInfo
     * @throws \Rest\RestException
     */
    public function fetchFirstLevelResellerAccountById($accountId)
    {
        $account = $this->_openApi->getAccountInfo($accountId);
        if (!$account
            || $account->getAccountType() !== Service_OpenApi_ApiInterface::ACCOUNT_TYPE_RESELLER
        ) {
            return null;
        }
        $parentAccount = $this->_openApi->getAccountInfo($account->getParentAccountId());
        if (!$parentAccount
            || $parentAccount->getAccountType() !== Service_OpenApi_ApiInterface::ACCOUNT_TYPE_PROVIDER
        ) {
            return null;
        }
        return $account;
    }

    /**
     * @param Service_PbaApi_Api_Response_Struct_DiscountInterface $discount
     * @return bool
     */
    public function detectAllowingDiscountForFirstLevelReseller(
        Service_PbaApi_Api_Response_Struct_DiscountInterface $discount
    ) {
        return floor($discount->getRate()) == self::REQUIRED_DISCOUNT_RATE_FOR_FIRST_LEVEL_RESELLER
            && $discount->getStatus() == 1
            && !$discount->getIsForAllPlans()
            && !$discount->getIsForAllResources()
            && $discount->getType() == 0;
    }

    /**
     * @return Service_OpenApi_Response_AccountInfo[]
     */
    public function fetchFirstLevelResellersAccounts()
    {
        $customers = $this->_openApi->getCustomerIds(Service_OpenApi_ApiInterface::ROOT_ACCOUNT_ID);
        $accounts = array();
        foreach ($customers as $customer) {
            $account = $this->_openApi->getAccountInfo($customer['account_id']);
            if ($account->getAccountType() !== Service_OpenApi_ApiInterface::ACCOUNT_TYPE_RESELLER) {
                continue;
            }
            $accounts []= $account;
        }
        return $accounts;
    }
}
