<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

use Acronis\Cloud\Client\Model\Common\Types\Role;

class Model_PropertyConverters_ServiceRolesConverter
{
    /** @noinspection PhpDocMissingThrowsInspection */
    /**
     * @param $userResource $user
     * @param bool $isBackupApplicationEnabledForTenant
     * @param bool $isNotaryApplicationEnabledForTenant
     * @return string[]
     */
    public static function getRoles($userResource, $applications = [])
    {
        $oldBackupPrivileges = Arr::get($userResource, 'privileged', 0);
        $tenantType = Arr::getNestedItem($userResource, ['tenant', 'type'], tenant::TYPE_CUSTOMER);

        $roles = [];
        if ($applications) {
            foreach ($applications as $application) {
                $role = Arr::get(users::APPLICATION_USER_ROLE_MAP, $application);
                if ($role && !in_array($role, $roles)) {
                    array_push($roles, $role);
                }
            }
        }

        if ($oldBackupPrivileges === users::ADMINISTRATOR_PRIVILEGES_NO) {
            return $roles;
        } else if ($tenantType == tenant::TYPE_CUSTOMER) {
            return array_merge($roles, [Role::COMPANY_ADMIN]);
        }

        return [Role::PARTNER_ADMIN];
    }
}