<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Services\Database;

use Acronis\App\Services\Config\ConfigAwareTrait;
use Acronis\App\Services\Config\ConfigSchema;
use Acronis\App\Services\FactoryInterface;
use Illuminate\Database\Capsule\Manager as Capsule;

class DatabaseFactory implements FactoryInterface
{
    use ConfigAwareTrait;

    const NAME = 'database';

    const DRIVER_SQLITE = 'sqlite';

    /**
     * @return \Illuminate\Database\Connection
     * @throws \Exception
     */
    public function createInstance()
    {
        $capsule = new Capsule;
        $databaseConfig = $this->getDatabaseConfig();

        if ($this->isSqlite($databaseConfig)) {
            $this->prepareSqlite($databaseConfig);
        }

        $capsule->addConnection($databaseConfig);
        $capsule->setAsGlobal();
        $capsule->bootEloquent();

        return $capsule->getConnection();
    }

    /**
     * @return mixed
     * @throws \Exception
     */
    private function getDatabaseConfig()
    {
        $config = $this->getConfig();
        $databaseConfig = $config->getDatabaseConfig();

        if ($this->isSqlite($databaseConfig)) {
            $databaseConfig[ConfigSchema::NODE_DATABASE_PATH] = $config->getAbsolutePath($databaseConfig[ConfigSchema::NODE_DATABASE_PATH]);
        }

        return $databaseConfig;
    }

    private function isSqlite($databaseConfig)
    {
        return $databaseConfig[ConfigSchema::NODE_DATABASE_DRIVER] === self::DRIVER_SQLITE;
    }

    private function prepareSqlite($sqliteConfig)
    {
        $filePath = $sqliteConfig[ConfigSchema::NODE_DATABASE_PATH];

        if (file_exists($filePath)) {
            return;
        }

        touch($filePath);
    }
}