<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\UsageReport;

use Acronis\App\Services\Redis\RedisClient;
use Json;
use Log;
use Monolog\Logger;
use Predis\Client;
use Time;

class RedisStorage implements ReportStorageInterface
{
    /** @var Client */
    private $redisClient;

    /** @var string */
    private $keyPrefix;

    /** @var int */
    private $ttl;

    /**
     * RedisStorage constructor.
     * @param RedisClient $redisClient
     * @param string $keyPrefix
     * @param int $ttl Expiration time in seconds
     */
    function __construct($redisClient, $keyPrefix = '', $ttl = null)
    {
        $this->redisClient = $redisClient;
        $this->keyPrefix = $keyPrefix;
        $this->ttl = $ttl;
    }

    /**
     * @param string $key
     * @param mixed $value
     * @throws \Exception
     */
    public function set($key, $value)
    {
        Log::getInstance()->additional(Logger::INFO, sprintf('Setting Redis key "%s" with value "%s" will expire at "%s".',
            $this->formatKey($key), $this->encode($value), date(Time::DEFAULT_DATE_TIME_FORMAT, time() + $this->ttl)
        ));        $this->redisClient->set($this->formatKey($key), $this->encode($value), $this->ttl);
    }

    /**
     * @param string $key
     * @return mixed|null
     * @throws \Exception
     */
    public function get($key)
    {
        $dataStr = $this->redisClient->get($this->formatKey($key));
        return !is_null($dataStr) ? $this->decode($dataStr) : null;
    }

    public function deleteAll($keyPrefix = '')
    {
        $this->redisClient->deleteAll($this->formatKey($keyPrefix));
    }

    /***
     * @param $data
     * @throws \Exception
     * @return false|string
     */
    protected function encode($data)
    {
        return serialize($data);
    }

    /**
     * @param $dataStr
     * @throws \Exception
     * @return mixed
     */
    protected function decode($dataStr)
    {
        return unserialize($dataStr);
    }

    /**
     * @param string $key
     * @return string
     */
    protected function formatKey($key)
    {
        return $this->keyPrefix . $key;
    }
}