<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

namespace Acronis\Common\CodeGenerator\OfferingItems;

use Acronis\Common\CodeGenerator\InterpolatorInterface;
use Twig\Environment;
use Twig\Loader\ArrayLoader;
use Twig\TwigFilter;

class RequirementsInterpolator implements InterpolatorInterface
{
    private $customFilters = [];

    /**
     * @param string $template
     * @param array $oi
     * @param array $additional
     * @return string
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function interpolate(string $template, $item, $additional): string
    {
        // To do do not create new TWIG instance each time, try to prepare one per template
        // or even full set of twig templates and use more efficient way
        $loader = new ArrayLoader([
            'template' => $template
        ]);
        $twig = new Environment($loader, ['autoescape' => false, 'optimizations' => 0]);
        foreach ($this->customFilters as $filter) {
            $twig->addFilter($filter);
        }

        $scope = ['item' => $item, 'additional' => $additional];

        return $twig->render('template', $scope);
    }

    /**
     * PhpInterpolator constructor.
     */
    public function __construct()
    {
        $this->customFilters[] = new TwigFilter('counterName', function($item) {
            $counterName = str_replace('_', '', ucwords($item['Offering item name'], '_'));

            if ($item['Measurement unit'] === 'quantity') {
                $counterName .= 'Count';
            }

            if ($item['Measurement unit'] === 'bytes' &&
                !str_contains($item['Offering item name'], 'child_storages')
            ) {
                $counterName .= 'Usage';
            }

            return $counterName;
        });

        $this->customFilters[] = new TwigFilter('getSchema', function($item) {
            // return 'offering_item_for_usage';

            if ($item['Measurement unit'] === 'bytes') {
                if (strpos($item['Offering item name'], 'dr_storage') !== false) {
                    return 'dr_storage_offering_item';
                }

                if (strpos($item['Offering item name'], 'fc_storage') !== false) {
                    return 'fc_storage_offering_item';
                }

                return 'storage_offering_item';
            }

            return 'offering_item';
        });
    }
}