<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

namespace Acronis\Common\CodeGenerator\Validators;

use Acronis\Common\CodeGenerator\Exception\OfferingItemValidationException;
use Symfony\Component\Validator\Validation;

abstract class AbstractValidator
{
    private $data;

    public function __construct($data)
    {
        $this->data = $data;
        foreach ($data as $key => $value) {
            if (property_exists($this, $key)) {
                $this->{$key} = $value;
            }
        }
    }

    public function validate()
    {
        $validator = Validation::createValidatorBuilder()
            ->enableAnnotationMapping()
            ->getValidator();

        $violations = $validator->validate($this);
        if (0 !== count($violations)) {
            // there are errors, now you can show them
            foreach ($violations as $violation) {
                error_log(sprintf(
                        'Property "%s" validation error: %s', $violation->getPropertyPath(), $violation->getMessage()
                    ) . PHP_EOL);
            }

            throw new OfferingItemValidationException($this->data);
        }
    }
}