<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

namespace Acronis\Common\Commands;

use Acronis\Common\CodeGenerator\CodeGenerator;
use Acronis\Common\CodeGenerator\Exception\CanNotRollbackFileException;
use Acronis\Common\Utils\CodegenHelper;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class Rollback extends AssetAwareCommand
{
    public function getName()
    {
        return 'code:rollback';
    }

    public function getDescription()
    {
        return 'Rollback generated code from .orig files.';
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $config = $this->loadAssetConfig($input);
        $templatesPath = realpath($config['base_path'] . $config['templates']);
        $codePath = realpath($config['code_base_path']);
        $codeGen = new CodeGenerator(new CodegenHelper($templatesPath, $codePath,false, false));
        try {
            $codeGen->rollback();
            echo sprintf('[CODEGEN TOOL] Generated code rolled back to previous state.');
        } catch (CanNotRollbackFileException $e) {
            echo sprintf('[CODEGEN TOOL] Unable to rollback code: "%s".', $e->getMessage());
        }

        return 0;
    }
}