<?php


use Acronis\Common\CodeGenerator\OfferingItems\OfferingItemInterpolator;

class OfferingItemsInterpolatorTest extends \PHPUnit\Framework\TestCase
{
    /**
     * @dataProvider templatesDataProvider
     */
    public function testInterpolate($template, $oi, $additional, $expected)
    {
        $interpolator = new OfferingItemInterpolator();

        $actual = $interpolator->interpolate($template, $oi, $additional);

        $this->assertEquals($expected, $actual);
    }

    public function templatesDataProvider()
    {
        return [
            'Simple interpolation' => [
                'template' => 'Hello, {{ oi.name }}!',
                'oi' => ['name' => 'World'],
                'additional' => null,
                'expected' => 'Hello, World!'
            ],
            'Undefined field' => [
                'template' => 'Hello, {{ oi.name2 }}!',
                'oi' => ['name' => 'World'],
                'additional' => null,
                'expected' => 'Hello, !'
            ],
            'Additional context' => [
                'template' => '{{ additional.verb }}, {{ oi.name }}!',
                'oi' => ['name' => 'World'],
                'additional' => ['verb' => 'Hello'],
                'expected' => 'Hello, World!'
            ],
            '"customUnderscore" filter' => [
                'template' => 'Hello, {{ oi.name | customUnderscore }}!',
                'oi' => ['name' => 'PhpWorld'],
                'additional' => null,
                'expected' => 'Hello, php_world!'
            ],
            '"customJsonWhenArrayOnly" filter on array' => [
                'template' => 'Hello, {{ oi | customJsonWhenArrayOnly }}!',
                'oi' => ['name' => 'world'],
                'additional' => null,
                'expected' => 'Hello, {"name":"world"}!'
            ],
            '"customJsonWhenArrayOnly" filter on string (not array)' => [
                'template' => 'Hello, {{ oi.name | customJsonWhenArrayOnly }}!',
                'oi' => ['name' => 'world'],
                'additional' => null,
                'expected' => 'Hello, world!'
            ]
        ];
    }
}